@@ -1,15 +1,15 @@
 /*
  * Copyright (C) 2004-2005 by Objective Systems, Inc.
  *
- * This software is furnished under an open source license and may be 
- * used and copied only in accordance with the terms of this license. 
- * The text of the license may generally be found in the root 
- * directory of this installation in the COPYING file.  It 
+ * This software is furnished under an open source license and may be
+ * used and copied only in accordance with the terms of this license.
+ * The text of the license may generally be found in the root
+ * directory of this installation in the COPYING file.  It
  * can also be viewed online at the following URL:
  *
  *   http://www.obj-sys.com/open/license.html
  *
- * Any redistributions of this file including modified versions must 
+ * Any redistributions of this file including modified versions must
  * maintain this copyright notice.
  *
  *****************************************************************************/
@@ -28,6 +28,7 @@
 #include "ooGkClient.h"
 #include "ooh323ep.h"
 #include "ooCalls.h"
+#include "ooCmdChannel.h"
 
 /** Global endpoint structure */
 extern OOH323EndPoint gH323ep;
@@ -60,7 +61,7 @@ OOH323CallData* ooCreateCall(char* type, char*callToken)
    {
       OOTRACEERR1("ERROR:Memory - ooCreateCall - call\n");
       return NULL;
-   } 
+   }
    memset(call, 0, sizeof(OOH323CallData));
    ast_cond_init(&call->gkWait, NULL);
    ast_mutex_init(&call->GkLock);
@@ -78,7 +79,7 @@ OOH323CallData* ooCreateCall(char* type, char*callToken)
    else {
       call->ourCallerId[0] = '\0';
    }
-   
+
    memset(&call->callIdentifier, 0, sizeof(H225CallIdentifier));
    memset(&call->confIdentifier, 0, sizeof(H225ConferenceIdentifier));
 
@@ -105,7 +106,7 @@ OOH323CallData* ooCreateCall(char* type, char*callToken)
 // May 20090713. Fix it for Video session
 
    OO_SETFLAG(call->flags, OO_M_AUDIOSESSION);
-   
+
    call->callState = OO_CALL_CREATED;
    call->callEndReason = OO_REASON_UNKNOWN;
    call->pCallFwdData = NULL;
@@ -114,13 +115,13 @@ OOH323CallData* ooCreateCall(char* type, char*callToken)
    {
       call->callingPartyNumber = NULL;
    }
-   else{      
+   else{
       if(ooUtilsIsStrEmpty(gH323ep.callingPartyNumber))
       {
          call->callingPartyNumber = NULL;
       }
       else{
-         call->callingPartyNumber = (char*) memAlloc(call->pctxt, 
+         call->callingPartyNumber = (char*) memAlloc(call->pctxt,
                                          strlen(gH323ep.callingPartyNumber)+1);
          if(call->callingPartyNumber)
          {
@@ -154,14 +155,14 @@ OOH323CallData* ooCreateCall(char* type, char*callToken)
    call->masterSlaveState = OO_MasterSlave_Idle;
    call->statusDeterminationNumber = 0;
    call->localTermCapState = OO_LocalTermCapExchange_Idle;
-   call->remoteTermCapState = OO_RemoteTermCapExchange_Idle; 
+   call->remoteTermCapState = OO_RemoteTermCapExchange_Idle;
    call->ourCaps = NULL;
    call->remoteCaps = NULL;
    call->jointCaps = NULL;
    dListInit(&call->remoteFastStartOLCs);
    call->remoteTermCapSeqNo =0;
    call->localTermCapSeqNo = 0;
-   memcpy(&call->capPrefs, &gH323ep.capPrefs, sizeof(OOCapPrefs));    
+   memcpy(&call->capPrefs, &gH323ep.capPrefs, sizeof(OOCapPrefs));
    call->logicalChans = NULL;
    call->noOfLogicalChannels = 0;
    call->logicalChanNoBase = 1001;
@@ -172,7 +173,8 @@ OOH323CallData* ooCreateCall(char* type, char*callToken)
    call->msdRetries = 0;
    call->pFastStartRes = NULL;
    call->usrData = NULL;
-   OOTRACEINFO3("Created a new call (%s, %s)\n", call->callType, 
+   ooCreateCallCmdConnection(call);
+   OOTRACEINFO3("Created a new call (%s, %s)\n", call->callType,
                  call->callToken);
    /* Add new call to calllist */
    ooAddCallToList (call);
@@ -206,8 +208,8 @@ int ooAddCallToList(OOH323CallData *call)
 
 int ooEndCall(OOH323CallData *call)
 {
-   OOTRACEDBGA4("In ooEndCall call state is - %s (%s, %s)\n", 
-                 ooGetCallStateText(call->callState), call->callType, 
+   OOTRACEDBGA4("In ooEndCall call state is - %s (%s, %s)\n",
+                 ooGetCallStateText(call->callState), call->callType,
                  call->callToken);
 
    if(call->callState == OO_CALL_REMOVED) {
@@ -226,7 +228,7 @@ int ooEndCall(OOH323CallData *call)
    if(call->callState == OO_CALL_CLEARED || ((strcmp(call->callType, "incoming")) &&
      call->callState == OO_CALL_CLEAR_RELEASESENT))
    {
-      ooCleanCall(call); 
+      ooCleanCall(call);
       call->callState = OO_CALL_REMOVED;
       return OO_OK;
    }
@@ -249,16 +251,16 @@ int ooEndCall(OOH323CallData *call)
    }
 
 
-   if(!OO_TESTFLAG(call->flags, OO_M_RELEASE_BUILT))   
+   if(!OO_TESTFLAG(call->flags, OO_M_RELEASE_BUILT))
    {
-     if(call->callState == OO_CALL_CLEAR || 
+     if(call->callState == OO_CALL_CLEAR ||
         call->callState == OO_CALL_CLEAR_RELEASERECVD)
      {
         ooSendReleaseComplete(call);
         OO_SETFLAG(call->flags, OO_M_RELEASE_BUILT);
      }
    }
-      
+
    return OO_OK;
 }
 
@@ -297,8 +299,8 @@ int ooCleanCall(OOH323CallData *call)
 {
    OOCTXT *pctxt;
 
-   OOTRACEWARN4 ("Cleaning Call (%s, %s)- reason:%s\n", 
-                 call->callType, call->callToken, 
+   OOTRACEWARN4 ("Cleaning Call (%s, %s)- reason:%s\n",
+                 call->callType, call->callToken,
                  ooGetReasonCodeText (call->callEndReason));
 
    /* First clean all the logical channels, if not already cleaned. */
@@ -321,7 +323,7 @@ int ooCleanCall(OOH323CallData *call)
    {
       ooCloseH245Listener(call);
    }
-   
+
    /* Close H225 connection, if not already closed. */
    if (0 != call->pH225Channel && 0 != call->pH225Channel->sock)
    {
@@ -340,7 +342,7 @@ int ooCleanCall(OOH323CallData *call)
    }
 
    ooRemoveCallFromList (call);
-   OOTRACEINFO3("Removed call (%s, %s) from list\n", call->callType, 
+   OOTRACEINFO3("Removed call (%s, %s) from list\n", call->callType,
                  call->callToken);
 
    if(call->pCallFwdData && call->pCallFwdData->fwdedByRemote)
@@ -393,7 +395,7 @@ int ooCallSetCallerId(OOH323CallData* call, const char* callerid)
 
 int ooCallSetCallingPartyNumber(OOH323CallData *call, const char *number)
 {
-   if(call->callingPartyNumber) 
+   if(call->callingPartyNumber)
       memFreePtr(call->pctxt, call->callingPartyNumber);
 
    call->callingPartyNumber = (char*) memAlloc(call->pctxt, strlen(number)+1);
@@ -403,7 +405,7 @@ int ooCallSetCallingPartyNumber(OOH323CallData *call, const char *number)
    }
    else{
       OOTRACEERR3("Error:Memory - ooCallSetCallingPartyNumber - "
-                  "callingPartyNumber.(%s, %s)\n", call->callType, 
+                  "callingPartyNumber.(%s, %s)\n", call->callType,
                   call->callToken);
       return OO_FAILED;
    }
@@ -425,14 +427,14 @@ int ooCallGetCallingPartyNumber(OOH323CallData *call, char *buffer, int len)
          return OO_OK;
       }
    }
-   
+
    return OO_FAILED;
 }
 
 
 int ooCallSetCalledPartyNumber(OOH323CallData *call, const char *number)
 {
-   if(call->calledPartyNumber) 
+   if(call->calledPartyNumber)
       memFreePtr(call->pctxt, call->calledPartyNumber);
 
    call->calledPartyNumber = (char*) memAlloc(call->pctxt, strlen(number)+1);
@@ -442,7 +444,7 @@ int ooCallSetCalledPartyNumber(OOH323CallData *call, const char *number)
    }
    else{
       OOTRACEERR3("Error:Memory - ooCallSetCalledPartyNumber - "
-                  "calledPartyNumber.(%s, %s)\n", call->callType, 
+                  "calledPartyNumber.(%s, %s)\n", call->callType,
                   call->callToken);
       return OO_FAILED;
    }
@@ -459,7 +461,7 @@ int ooCallGetCalledPartyNumber(OOH323CallData *call, char *buffer, int len)
          return OO_OK;
       }
    }
-   
+
    return OO_FAILED;
 }
 
@@ -504,7 +506,7 @@ int ooCallAddAlias
      call->remoteAliases = psNewAlias;
    }
 
-   OOTRACEDBGC5("Added %s alias %s to call. (%s, %s)\n", 
+   OOTRACEDBGC5("Added %s alias %s to call. (%s, %s)\n",
               local?"local":"remote", value, call->callType, call->callToken);
    return OO_OK;
 }
@@ -531,8 +533,8 @@ int ooCallAddAliasEmailID(OOH323CallData *call, const char* email)
 int ooCallAddAliasURLID(OOH323CallData *call, const char* url)
 {
    return ooCallAddAlias(call, T_H225AliasAddress_url_ID, url, TRUE);
-} 
- 
+}
+
 
 int ooCallAddRemoteAliasH323ID(OOH323CallData *call, const char* h323id)
 {
@@ -548,63 +550,63 @@ int ooCallAddRemoteAliasDialedDigits
 
 
 
-/* Used to override global end point capabilities and add call specific 
+/* Used to override global end point capabilities and add call specific
    capabilities */
 
-int ooCallAddG726Capability(OOH323CallData *call, int cap, int txframes, 
+int ooCallAddG726Capability(OOH323CallData *call, int cap, int txframes,
                             int rxframes, OOBOOL silenceSuppression, int dir,
                             cb_StartReceiveChannel startReceiveChannel,
                             cb_StartTransmitChannel startTransmitChannel,
                             cb_StopReceiveChannel stopReceiveChannel,
                             cb_StopTransmitChannel stopTransmitChannel)
 {
-   return ooCapabilityAddSimpleCapability(call, cap, txframes, rxframes, 
-                                silenceSuppression, dir, startReceiveChannel, 
-                                startTransmitChannel, stopReceiveChannel, 
+   return ooCapabilityAddSimpleCapability(call, cap, txframes, rxframes,
+                                silenceSuppression, dir, startReceiveChannel,
+                                startTransmitChannel, stopReceiveChannel,
                                 stopTransmitChannel, FALSE);
 }
-int ooCallAddAMRNBCapability(OOH323CallData *call, int cap, int txframes, 
+int ooCallAddAMRNBCapability(OOH323CallData *call, int cap, int txframes,
                             int rxframes, OOBOOL silenceSuppression, int dir,
                             cb_StartReceiveChannel startReceiveChannel,
                             cb_StartTransmitChannel startTransmitChannel,
                             cb_StopReceiveChannel stopReceiveChannel,
                             cb_StopTransmitChannel stopTransmitChannel)
 {
-   return ooCapabilityAddSimpleCapability(call, cap, txframes, rxframes, 
-                                silenceSuppression, dir, startReceiveChannel, 
-                                startTransmitChannel, stopReceiveChannel, 
+   return ooCapabilityAddSimpleCapability(call, cap, txframes, rxframes,
+                                silenceSuppression, dir, startReceiveChannel,
+                                startTransmitChannel, stopReceiveChannel,
                                 stopTransmitChannel, FALSE);
 }
 
-int ooCallAddSpeexCapability(OOH323CallData *call, int cap, int txframes, 
+int ooCallAddSpeexCapability(OOH323CallData *call, int cap, int txframes,
                             int rxframes, OOBOOL silenceSuppression, int dir,
                             cb_StartReceiveChannel startReceiveChannel,
                             cb_StartTransmitChannel startTransmitChannel,
                             cb_StopReceiveChannel stopReceiveChannel,
                             cb_StopTransmitChannel stopTransmitChannel)
 {
-   return ooCapabilityAddSimpleCapability(call, cap, txframes, rxframes, 
-                                silenceSuppression, dir, startReceiveChannel, 
-                                startTransmitChannel, stopReceiveChannel, 
+   return ooCapabilityAddSimpleCapability(call, cap, txframes, rxframes,
+                                silenceSuppression, dir, startReceiveChannel,
+                                startTransmitChannel, stopReceiveChannel,
                                 stopTransmitChannel, FALSE);
 }
 
-int ooCallAddG7231Capability(OOH323CallData *call, int cap, int txframes, 
+int ooCallAddG7231Capability(OOH323CallData *call, int cap, int txframes,
                             int rxframes, OOBOOL silenceSuppression, int dir,
                             cb_StartReceiveChannel startReceiveChannel,
                             cb_StartTransmitChannel startTransmitChannel,
                             cb_StopReceiveChannel stopReceiveChannel,
                             cb_StopTransmitChannel stopTransmitChannel)
 {
-   return ooCapabilityAddSimpleCapability(call, cap, txframes, rxframes, 
-                                silenceSuppression, dir, startReceiveChannel, 
-                                startTransmitChannel, stopReceiveChannel, 
+   return ooCapabilityAddSimpleCapability(call, cap, txframes, rxframes,
+                                silenceSuppression, dir, startReceiveChannel,
+                                startTransmitChannel, stopReceiveChannel,
                                 stopTransmitChannel, FALSE);
 }
 
 
 
-int ooCallAddG729Capability(OOH323CallData *call, int cap, int txframes, 
+int ooCallAddG729Capability(OOH323CallData *call, int cap, int txframes,
                             int rxframes, int dir,
                             cb_StartReceiveChannel startReceiveChannel,
                             cb_StartTransmitChannel startTransmitChannel,
@@ -612,12 +614,12 @@ int ooCallAddG729Capability(OOH323CallData *call, int cap, int txframes,
                             cb_StopTransmitChannel stopTransmitChannel)
 {
    return ooCapabilityAddSimpleCapability(call, cap, txframes, rxframes, FALSE,
-                          dir, startReceiveChannel, startTransmitChannel, 
+                          dir, startReceiveChannel, startTransmitChannel,
                           stopReceiveChannel, stopTransmitChannel, FALSE);
 }
 
 /*
-int ooCallAddG726Capability(OOH323CallData *call, int cap, int txframes, 
+int ooCallAddG726Capability(OOH323CallData *call, int cap, int txframes,
                             int rxframes, int dir,
                             cb_StartReceiveChannel startReceiveChannel,
                             cb_StartTransmitChannel startTransmitChannel,
@@ -625,12 +627,12 @@ int ooCallAddG726Capability(OOH323CallData *call, int cap, int txframes,
                             cb_StopTransmitChannel stopTransmitChannel)
 {
    return ooCapabilityAddSimpleCapability(call, cap, txframes, rxframes, FALSE,
-                          dir, startReceiveChannel, startTransmitChannel, 
+                          dir, startReceiveChannel, startTransmitChannel,
                           stopReceiveChannel, stopTransmitChannel, FALSE);
 }
 */
 
-int ooCallAddG728Capability(OOH323CallData *call, int cap, int txframes, 
+int ooCallAddG728Capability(OOH323CallData *call, int cap, int txframes,
                             int rxframes, int dir,
                             cb_StartReceiveChannel startReceiveChannel,
                             cb_StartTransmitChannel startTransmitChannel,
@@ -638,11 +640,11 @@ int ooCallAddG728Capability(OOH323CallData *call, int cap, int txframes,
                             cb_StopTransmitChannel stopTransmitChannel)
 {
    return ooCapabilityAddSimpleCapability(call, cap, txframes, rxframes, FALSE,
-                          dir, startReceiveChannel, startTransmitChannel, 
+                          dir, startReceiveChannel, startTransmitChannel,
                           stopReceiveChannel, stopTransmitChannel, FALSE);
 }
 
-int ooCallAddG711Capability(OOH323CallData *call, int cap, int txframes, 
+int ooCallAddG711Capability(OOH323CallData *call, int cap, int txframes,
                             int rxframes, int dir,
                             cb_StartReceiveChannel startReceiveChannel,
                             cb_StartTransmitChannel startTransmitChannel,
@@ -650,28 +652,28 @@ int ooCallAddG711Capability(OOH323CallData *call, int cap, int txframes,
                             cb_StopTransmitChannel stopTransmitChannel)
 {
    return ooCapabilityAddSimpleCapability(call, cap, txframes, rxframes, FALSE,
-                            dir, startReceiveChannel, startTransmitChannel, 
+                            dir, startReceiveChannel, startTransmitChannel,
                             stopReceiveChannel, stopTransmitChannel, FALSE);
 }
 
 int ooCallAddGSMCapability
-   (OOH323CallData* call, int cap, ASN1USINT framesPerPkt, 
+   (OOH323CallData* call, int cap, ASN1USINT framesPerPkt,
     OOBOOL comfortNoise, OOBOOL scrambled, int dir,
     cb_StartReceiveChannel startReceiveChannel,
     cb_StartTransmitChannel startTransmitChannel,
     cb_StopReceiveChannel stopReceiveChannel,
     cb_StopTransmitChannel stopTransmitChannel)
 {
-   return ooCapabilityAddGSMCapability(call, cap, framesPerPkt, comfortNoise, 
-                                     scrambled, dir, startReceiveChannel, 
+   return ooCapabilityAddGSMCapability(call, cap, framesPerPkt, comfortNoise,
+                                     scrambled, dir, startReceiveChannel,
                                      startTransmitChannel, stopReceiveChannel,
                                      stopTransmitChannel, FALSE);
 }
 
 
 int ooCallAddH263VideoCapability
-   (OOH323CallData *call, int cap, unsigned sqcifMPI, unsigned qcifMPI, 
-    unsigned cifMPI, unsigned cif4MPI, unsigned cif16MPI, unsigned maxBitRate, 
+   (OOH323CallData *call, int cap, unsigned sqcifMPI, unsigned qcifMPI,
+    unsigned cifMPI, unsigned cif4MPI, unsigned cif16MPI, unsigned maxBitRate,
     int dir, cb_StartReceiveChannel startReceiveChannel,
     cb_StartTransmitChannel startTransmitChannel,
     cb_StopReceiveChannel stopReceiveChannel,
@@ -681,7 +683,7 @@ int ooCallAddH263VideoCapability
    return ooCapabilityAddH263VideoCapability(call, sqcifMPI, qcifMPI, cifMPI,
                                      cif4MPI, cif16MPI, maxBitRate,dir,
                                      startReceiveChannel, startTransmitChannel,
-                                     stopReceiveChannel, stopTransmitChannel, 
+                                     stopReceiveChannel, stopTransmitChannel,
                                      FALSE);
 
 }
@@ -763,7 +765,7 @@ OOH323CallData* ooFindCallByToken(const char *callToken)
       else
          call = call->next;
    }
-   
+
    if(!call)
    {
       OOTRACEERR2("ERROR:Call with token %s not found\n", callToken);
@@ -788,7 +790,7 @@ ASN1BOOL ooIsSessionEstablished(OOH323CallData *call, int sessionID, char* dir)
    while(temp)
    {
       if(temp->sessionID == sessionID              &&
-         temp->state == OO_LOGICALCHAN_ESTABLISHED && 
+         temp->state == OO_LOGICALCHAN_ESTABLISHED &&
          !strcmp(temp->dir, dir)                     )
          return TRUE;
       temp = temp->next;
@@ -802,8 +804,7 @@ int ooAddMediaInfo(OOH323CallData *call, OOMediaInfo mediaInfo)
 
    if(!call)
    {
-      OOTRACEERR3("Error:Invalid 'call' param for ooAddMediaInfo.(%s, %s)\n",
-                   call->callType, call->callToken);
+      OOTRACEERR1("Error:Invalid 'call' param for ooAddMediaInfo.\n");
       return OO_FAILED;
    }
    newMediaInfo = (OOMediaInfo*) memAlloc(call->pctxt, sizeof(OOMediaInfo));
@@ -816,7 +817,7 @@ int ooAddMediaInfo(OOH323CallData *call, OOMediaInfo mediaInfo)
 
    memcpy (newMediaInfo, &mediaInfo, sizeof(OOMediaInfo));
 
-   OOTRACEDBGC4("Configured mediainfo for cap %s (%s, %s)\n", 
+   OOTRACEDBGC4("Configured mediainfo for cap %s (%s, %s)\n",
                 ooGetCapTypeText(mediaInfo.cap),
                 call->callType, call->callToken);
    if(!call->mediaInfo) {
@@ -846,7 +847,7 @@ unsigned ooCallGenerateSessionID
             sessionID = call->nextSessionID++;
          else{
             OOTRACEDBGC4("Session id for %s channel of type audio has to be "
-                        "provided by remote.(%s, %s)\n", dir, call->callType, 
+                        "provided by remote.(%s, %s)\n", dir, call->callType,
                          call->callToken);
             sessionID = 0; /* Will be assigned by remote */
          }
@@ -865,7 +866,7 @@ unsigned ooCallGenerateSessionID
          else{
             sessionID = 0; /* Will be assigned by remote */
             OOTRACEDBGC4("Session id for %s channel of type video has to be "
-                        "provided by remote.(%s, %s)\n", dir, call->callType, 
+                        "provided by remote.(%s, %s)\n", dir, call->callType,
                          call->callToken);
          }
       }
@@ -882,7 +883,7 @@ unsigned ooCallGenerateSessionID
          else{
             sessionID = 0; /* Will be assigned by remote */
             OOTRACEDBGC4("Session id for %s channel of type data has to be "
-                        "provided by remote.(%s, %s)\n", dir, call->callType, 
+                        "provided by remote.(%s, %s)\n", dir, call->callType,
                          call->callToken);
          }
       }
@@ -897,8 +898,8 @@ int ooCallH245ConnectionRetryTimerExpired(void *data)
    ooTimerCallback *cbData = (ooTimerCallback*) data;
    OOH323CallData *call = cbData->call;
 
-   OOTRACEINFO3("H245 connection retry timer expired. (%s, %s)\n", 
-                                            call->callType, call->callToken); 
+   OOTRACEINFO3("H245 connection retry timer expired. (%s, %s)\n",
+                                            call->callType, call->callToken);
    memFreePtr(call->pctxt, cbData);
 
    call->h245ConnectionAttempts++;
@@ -911,9 +912,9 @@ int ooCallH245ConnectionRetryTimerExpired(void *data)
 const char* ooGetReasonCodeText (OOUINT32 code)
 {
    static const char* reasonCodeText[] = {
-      "OO_REASON_UNKNOWN", 
+      "OO_REASON_UNKNOWN",
       "OO_REASON_INVALIDMESSAGE",
-      "OO_REASON_TRANSPORTFAILURE", 
+      "OO_REASON_TRANSPORTFAILURE",
       "OO_REASON_NOROUTE",
       "OO_REASON_NOUSER",
       "OO_REASON_NOBW",
@@ -923,10 +924,10 @@ const char* ooGetReasonCodeText (OOUINT32 code)
       "OO_REASON_GK_UNREACHABLE",
       "OO_REASON_GK_CLEARED",
       "OO_REASON_NOCOMMON_CAPABILITIES",
-      "OO_REASON_REMOTE_FWDED",   
+      "OO_REASON_REMOTE_FWDED",
       "OO_REASON_LOCAL_FWDED",
-      "OO_REASON_REMOTE_CLEARED", 
-      "OO_REASON_LOCAL_CLEARED", 
+      "OO_REASON_REMOTE_CLEARED",
+      "OO_REASON_LOCAL_CLEARED",
       "OO_REASON_REMOTE_BUSY",
       "OO_REASON_LOCAL_BUSY",
       "OO_REASON_REMOTE_NOANSWER",
@@ -954,4 +955,3 @@ const char* ooGetCallStateText (OOCallState callState)
    };
    return ooUtilsGetText (callState, callStateText, OONUMBEROF(callStateText));
 }
-
