@@ -32,7 +32,7 @@
  * \addtogroup configuration_file Configuration Files
  */
 
-/*! 
+/*!
  * \page amd.conf amd.conf
  * \verbinclude amd.conf.sample
  */
@@ -279,6 +279,12 @@ static void isAnsweringMachine(struct ast_channel *chan, const char *data)
 
 	/* Now we go into a loop waiting for frames from the channel */
 	while ((res = ast_waitfor(chan, 2 * maxWaitTimeForFrame)) > -1) {
+		int ms = 0;
+
+		/* Figure out how long we waited */
+		if (res > 0) {
+			ms = 2 * maxWaitTimeForFrame - res;
+		}
 
 		/* If we fail to read in a frame, that means they hung up */
 		if (!(f = ast_read(chan))) {
@@ -289,15 +295,22 @@ static void isAnsweringMachine(struct ast_channel *chan, const char *data)
 			break;
 		}
 
-		if (f->frametype == AST_FRAME_VOICE || f->frametype == AST_FRAME_NULL || f->frametype == AST_FRAME_CNG) {
-			/* If the total time exceeds the analysis time then give up as we are not too sure */
+		if (f->frametype == AST_FRAME_VOICE || f->frametype == AST_FRAME_CNG) {
+			/* Figure out how long the frame is in milliseconds */
 			if (f->frametype == AST_FRAME_VOICE) {
 				framelength = (ast_codec_samples_count(f) / DEFAULT_SAMPLES_PER_MS);
 			} else {
-				framelength = 2 * maxWaitTimeForFrame;
+				framelength = ms;
 			}
 
 			iTotalTime += framelength;
+
+			ast_debug(1, "AMD: Channel [%s] frametype [%s] iTotalTime [%d] framelength [%d] totalAnalysisTime [%d]\n",
+					  ast_channel_name(chan),
+					  f->frametype == AST_FRAME_VOICE ? "AST_FRAME_VOICE" : "AST_FRAME_CNG",
+					  iTotalTime, framelength, totalAnalysisTime);
+
+			/* If the total time exceeds the analysis time then give up as we are not too sure */
 			if (iTotalTime >= totalAnalysisTime) {
 				ast_verb(3, "AMD: Channel [%s]. Too long...\n", ast_channel_name(chan));
 				ast_frfree(f);
@@ -308,7 +321,7 @@ static void isAnsweringMachine(struct ast_channel *chan, const char *data)
 
 			/* Feed the frame of audio into the silence detector and see if we get a result */
 			if (f->frametype != AST_FRAME_VOICE)
-				dspsilence += 2 * maxWaitTimeForFrame;
+				dspsilence += framelength;
 			else {
 				dspsilence = 0;
 				ast_dsp_silence(silenceDetector, f, &dspsilence);
@@ -316,7 +329,7 @@ static void isAnsweringMachine(struct ast_channel *chan, const char *data)
 
 			if (dspsilence > 0) {
 				silenceDuration = dspsilence;
-				
+
 				if (silenceDuration >= betweenWordsSilence) {
 					if (currentState != STATE_IN_SILENCE ) {
 						ast_verb(3, "AMD: Channel [%s]. Changed state to STATE_IN_SILENCE\n", ast_channel_name(chan));
@@ -338,7 +351,7 @@ static void isAnsweringMachine(struct ast_channel *chan, const char *data)
 					res = 1;
 					break;
 				}
-				
+
 				if (silenceDuration >= afterGreetingSilence  &&  inGreeting == 1) {
 					ast_verb(3, "AMD: Channel [%s]. HUMAN: silenceDuration:%d afterGreetingSilence:%d\n",
 						ast_channel_name(chan), silenceDuration, afterGreetingSilence);
@@ -348,7 +361,7 @@ static void isAnsweringMachine(struct ast_channel *chan, const char *data)
 					res = 1;
 					break;
 				}
-				
+
 			} else {
 				consecutiveVoiceDuration += framelength;
 				voiceDuration += framelength;
@@ -397,12 +410,12 @@ static void isAnsweringMachine(struct ast_channel *chan, const char *data)
 					inInitialSilence = 0;
 					inGreeting = 1;
 				}
-				
+
 			}
 		}
 		ast_frfree(f);
 	}
-	
+
 	if (!res) {
 		/* It took too long to get a frame back. Giving up. */
 		ast_verb(3, "AMD: Channel [%s]. Too long...\n", ast_channel_name(chan));
@@ -507,8 +520,8 @@ static int unload_module(void)
  * Module loading including tests for configuration or dependencies.
  * This function can return AST_MODULE_LOAD_FAILURE, AST_MODULE_LOAD_DECLINE,
  * or AST_MODULE_LOAD_SUCCESS. If a dependency or environment variable fails
- * tests return AST_MODULE_LOAD_FAILURE. If the module can not load the 
- * configuration file or other non-critical problem return 
+ * tests return AST_MODULE_LOAD_FAILURE. If the module can not load the
+ * configuration file or other non-critical problem return
  * AST_MODULE_LOAD_DECLINE. On success return AST_MODULE_LOAD_SUCCESS.
  */
 static int load_module(void)
