@@ -49,9 +49,9 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 		</synopsis>
 		<syntax>
 			<parameter name="vm-context">
-				<para>This is the context within voicemail.conf to use for the Directory. If not 
-				specified and <literal>searchcontexts=no</literal> in 
-				<filename>voicemail.conf</filename>, then <literal>default</literal> 
+				<para>This is the context within voicemail.conf to use for the Directory. If not
+				specified and <literal>searchcontexts=no</literal> in
+				<filename>voicemail.conf</filename>, then <literal>default</literal>
 				will be assumed.</para>
 			</parameter>
 			<parameter name="dial-context" required="false">
@@ -107,7 +107,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 					</option>
 				</optionlist>
 				<note><para>Only one of the <replaceable>f</replaceable>, <replaceable>l</replaceable>, or <replaceable>b</replaceable>
-				options may be specified. <emphasis>If more than one is specified</emphasis>, then Directory will act as 
+				options may be specified. <emphasis>If more than one is specified</emphasis>, then Directory will act as
 				if <replaceable>b</replaceable> was specified.  The number
 				of characters for the user to type defaults to <literal>3</literal>.</para></note>
 
@@ -366,7 +366,7 @@ static int select_item_seq(struct ast_channel *chan, struct directory_item **ite
 			if (!res)
 				res = ast_waitfordigit(chan, 3000);
 			ast_stopstream(chan);
-	
+
 			if (res == '0') { /* operator selected */
 				goto_exten(chan, dialcontext, "o");
 				pbx_builtin_setvar_helper(chan, "DIRECTORY_RESULT", "OPERATOR");
@@ -817,10 +817,10 @@ static int do_directory(struct ast_channel *chan, struct ast_config *vmcfg, stru
 	/* Sort items */
 	sort_items(sorted, count);
 
-	if (option_debug) {
-		ast_debug(2, "Listing matching entries:\n");
+	if (DEBUG_ATLEAST(2)) {
+		ast_log(LOG_DEBUG, "Listing matching entries:\n");
 		for (ptr = sorted, i = 0; i < count; i++, ptr++) {
-			ast_debug(2, "%s: %s\n", ptr[0]->exten, ptr[0]->name);
+			ast_log(LOG_DEBUG, "%s: %s\n", ptr[0]->exten, ptr[0]->name);
 		}
 	}
 
@@ -934,7 +934,7 @@ static int directory_exec(struct ast_channel *chan, const char *data)
 				res = ast_stream_and_wait(chan, digits, AST_DIGIT_ANY);
 			}
 			if (!res) {
-				res = ast_stream_and_wait(chan, 
+				res = ast_stream_and_wait(chan,
 					which == FIRST ? "dir-first" :
 					which == LAST ? "dir-last" :
 					"dir-firstlast", AST_DIGIT_ANY);
