@@ -1789,3 +1789,79 @@ UPDATE alembic_version SET version_num='a1698e8bb9c5' WHERE alembic_version.vers
 
 /
 
+-- Running upgrade a1698e8bb9c5 -> 20abce6d1e3c
+
+ALTER TABLE ps_endpoints MODIFY identify_by VARCHAR(13 CHAR)
+
+/
+
+ALTER TABLE ps_endpoints ADD CONSTRAINT pjsip_identify_by_values CHECK (identify_by IN ('username', 'auth_username', 'ip'))
+
+/
+
+UPDATE alembic_version SET version_num='20abce6d1e3c' WHERE alembic_version.version_num = 'a1698e8bb9c5'
+
+/
+
+-- Running upgrade 20abce6d1e3c -> 52798ad97bdf
+
+ALTER TABLE ps_endpoints MODIFY identify_by VARCHAR2(80 CHAR)
+
+/
+
+UPDATE alembic_version SET version_num='52798ad97bdf' WHERE alembic_version.version_num = '20abce6d1e3c'
+
+/
+
+-- Running upgrade 52798ad97bdf -> d3e4284f8707
+
+ALTER TABLE ps_subscription_persistence ADD prune_on_boot VARCHAR(3 CHAR)
+
+/
+
+ALTER TABLE ps_subscription_persistence ADD CONSTRAINT yesno_values CHECK (prune_on_boot IN ('yes', 'no'))
+
+/
+
+UPDATE alembic_version SET version_num='d3e4284f8707' WHERE alembic_version.version_num = '52798ad97bdf'
+
+/
+
+-- Running upgrade d3e4284f8707 -> 0be05c3a8225
+
+ALTER TABLE ps_systems ADD follow_early_media_fork VARCHAR(3 CHAR)
+
+/
+
+ALTER TABLE ps_systems ADD CONSTRAINT yesno_values CHECK (follow_early_media_fork IN ('yes', 'no'))
+
+/
+
+ALTER TABLE ps_systems ADD accept_multiple_sdp_answers VARCHAR(3 CHAR)
+
+/
+
+ALTER TABLE ps_systems ADD CONSTRAINT yesno_values CHECK (accept_multiple_sdp_answers IN ('yes', 'no'))
+
+/
+
+ALTER TABLE ps_endpoints ADD follow_early_media_fork VARCHAR(3 CHAR)
+
+/
+
+ALTER TABLE ps_endpoints ADD CONSTRAINT yesno_values CHECK (follow_early_media_fork IN ('yes', 'no'))
+
+/
+
+ALTER TABLE ps_endpoints ADD accept_multiple_sdp_answers VARCHAR(3 CHAR)
+
+/
+
+ALTER TABLE ps_endpoints ADD CONSTRAINT yesno_values CHECK (accept_multiple_sdp_answers IN ('yes', 'no'))
+
+/
+
+UPDATE alembic_version SET version_num='0be05c3a8225' WHERE alembic_version.version_num = 'd3e4284f8707'
+
+/
+
