@@ -29,6 +29,7 @@
 
 /*** MODULEINFO
 	<depend>res_odbc</depend>
+	<depend>generic_odbc</depend>
 	<support_level>core</support_level>
  ***/
 
@@ -56,8 +57,8 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 			<parameter name="result-id" required="true" />
 		</syntax>
 		<description>
-			<para>For queries which are marked as mode=multirow, the original 
-			query returns a <replaceable>result-id</replaceable> from which results 
+			<para>For queries which are marked as mode=multirow, the original
+			query returns a <replaceable>result-id</replaceable> from which results
 			may be fetched.  This function implements the actual fetch of the results.</para>
 			<para>This also sets <variable>ODBC_FETCH_STATUS</variable>.</para>
 			<variablelist>
@@ -80,7 +81,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 			<parameter name="result-id" required="true" />
 		</syntax>
 		<description>
-			<para>For queries which are marked as mode=multirow, this will clear 
+			<para>For queries which are marked as mode=multirow, this will clear
 			any remaining rows of the specified resultset.</para>
 		</description>
 	</application>
@@ -92,7 +93,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 			<parameter name="string" required="true" />
 		</syntax>
 		<description>
-			<para>Used in SQL templates to escape data which may contain single ticks 
+			<para>Used in SQL templates to escape data which may contain single ticks
 			<literal>'</literal> which are otherwise used to delimit data.</para>
 			<para>Example: SELECT foo FROM bar WHERE baz='${SQL_ESC(${ARG1})}'</para>
 		</description>
@@ -793,6 +794,7 @@ static int acf_odbc_read(struct ast_channel *chan, const char *cmd, char *s, cha
 		if (!(resultset = ast_calloc(1, sizeof(*resultset)))) {
 			pbx_builtin_setvar_helper(chan, "ODBCROWS", rowcount);
 			pbx_builtin_setvar_helper(chan, "ODBCSTATUS", status);
+			AST_RWLIST_UNLOCK(&queries);
 			ast_autoservice_stop(chan);
 			return -1;
 		}
@@ -809,6 +811,7 @@ static int acf_odbc_read(struct ast_channel *chan, const char *cmd, char *s, cha
 			if (!(resultset = ast_calloc(1, sizeof(*resultset)))) {
 				pbx_builtin_setvar_helper(chan, "ODBCROWS", rowcount);
 				pbx_builtin_setvar_helper(chan, "ODBCSTATUS", status);
+				AST_RWLIST_UNLOCK(&queries);
 				ast_autoservice_stop(chan);
 				return -1;
 			}
@@ -856,6 +859,21 @@ static int acf_odbc_read(struct ast_channel *chan, const char *cmd, char *s, cha
 		return -1;
 	}
 
+	if (colcount <= 0) {
+		ast_verb(4, "Returned %d columns [%s]\n", colcount, ast_str_buffer(sql));
+		buf[0] = '\0';
+		SQLCloseCursor(stmt);
+		SQLFreeHandle (SQL_HANDLE_STMT, stmt);
+		release_obj_or_dsn (&obj, &dsn);
+		if (!bogus_chan) {
+			pbx_builtin_setvar_helper(chan, "ODBCROWS", "0");
+			pbx_builtin_setvar_helper(chan, "ODBCSTATUS", "NODATA");
+			ast_autoservice_stop(chan);
+		}
+		odbc_datastore_free(resultset);
+		return 0;
+	}
+
 	res = SQLFetch(stmt);
 	if ((res != SQL_SUCCESS) && (res != SQL_SUCCESS_WITH_INFO)) {
 		int res1 = -1;
@@ -1519,6 +1537,15 @@ static char *cli_odbc_read(struct ast_cli_entry *e, int cmd, struct ast_cli_args
 				return CLI_SUCCESS;
 			}
 
+			if (colcount <= 0) {
+				SQLCloseCursor(stmt);
+				SQLFreeHandle (SQL_HANDLE_STMT, stmt);
+				release_obj_or_dsn (&obj, &dsn);
+				ast_cli(a->fd, "Returned %d columns.  Query executed on handle %d:%s [%s]\n", colcount, dsn_num, query->readhandle[dsn_num], ast_str_buffer(sql));
+				AST_RWLIST_UNLOCK(&queries);
+				return CLI_SUCCESS;
+			}
+
 			res = SQLFetch(stmt);
 			if ((res != SQL_SUCCESS) && (res != SQL_SUCCESS_WITH_INFO)) {
 				SQLCloseCursor(stmt);
@@ -1919,4 +1946,3 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "ODBC lookups",
 		.unload = unload_module,
 		.reload = reload,
 	       );
-
