@@ -463,16 +463,17 @@ struct ast_str *ast_manager_build_channel_state_string_prefix(
 		const struct ast_channel_snapshot *snapshot,
 		const char *prefix)
 {
-	struct ast_str *out = ast_str_create(1024);
-	int res = 0;
-	char *caller_name, *connected_name;
+	struct ast_str *out;
+	char *caller_name;
+	char *connected_name;
+	int res;
 
-	if (!out) {
+	if (snapshot->tech_properties & AST_CHAN_TP_INTERNAL) {
 		return NULL;
 	}
 
-	if (snapshot->tech_properties & AST_CHAN_TP_INTERNAL) {
-		ast_free(out);
+	out = ast_str_create(1024);
+	if (!out) {
 		return NULL;
 	}
 
@@ -509,10 +510,11 @@ struct ast_str *ast_manager_build_channel_state_string_prefix(
 		prefix, snapshot->uniqueid,
 		prefix, snapshot->linkedid);
 
+	ast_free(caller_name);
+	ast_free(connected_name);
+
 	if (!res) {
 		ast_free(out);
-		ast_free(caller_name);
-		ast_free(connected_name);
 		return NULL;
 	}
 
@@ -528,9 +530,6 @@ struct ast_str *ast_manager_build_channel_state_string_prefix(
 		}
 	}
 
-	ast_free(caller_name);
-	ast_free(connected_name);
-
 	return out;
 }
 
@@ -1300,4 +1299,3 @@ int manager_channels_init(void)
 
 	return 0;
 }
-
