@@ -314,7 +314,7 @@
 				<parameter name="ReceivedChallenge" required="false">
 					<para>The challenge that was received.</para>
 				</parameter>
-				<parameter name="RecievedHash" required="false">
+				<parameter name="ReceivedHash" required="false">
 					<para>The hash that was received.</para>
 				</parameter>
 			</syntax>
@@ -1196,5 +1196,3 @@ int ast_security_event_report(const struct ast_security_event_common *sec)
 
 	return 0;
 }
-
-
