@@ -59,6 +59,8 @@ static void *sorcery_realtime_retrieve_fields(const struct ast_sorcery *sorcery,
 static void sorcery_realtime_retrieve_multiple(const struct ast_sorcery *sorcery, void *data, const char *type, struct ao2_container *objects,
 					     const struct ast_variable *fields);
 static void sorcery_realtime_retrieve_regex(const struct ast_sorcery *sorcery, void *data, const char *type, struct ao2_container *objects, const char *regex);
+static void sorcery_realtime_retrieve_prefix(const struct ast_sorcery *sorcery, void *data, const char *type,
+					     struct ao2_container *objects, const char *prefix, const size_t prefix_len);
 static int sorcery_realtime_update(const struct ast_sorcery *sorcery, void *data, void *object);
 static int sorcery_realtime_delete(const struct ast_sorcery *sorcery, void *data, void *object);
 static void sorcery_realtime_close(void *data);
@@ -71,6 +73,7 @@ static struct ast_sorcery_wizard realtime_object_wizard = {
 	.retrieve_fields = sorcery_realtime_retrieve_fields,
 	.retrieve_multiple = sorcery_realtime_retrieve_multiple,
 	.retrieve_regex = sorcery_realtime_retrieve_regex,
+	.retrieve_prefix = sorcery_realtime_retrieve_prefix,
 	.update = sorcery_realtime_update,
 	.delete = sorcery_realtime_delete,
 	.close = sorcery_realtime_close,
@@ -94,7 +97,7 @@ static int sorcery_realtime_create(const struct ast_sorcery *sorcery, void *data
 	return (ast_store_realtime_fields(config->family, fields) <= 0) ? -1 : 0;
 }
 
-/*! \brief Internal helper function which returns a filtered objectset. 
+/*! \brief Internal helper function which returns a filtered objectset.
  *
  * The following are filtered out of the objectset:
  * \li The id field. This is returned to the caller in an out parameter.
@@ -175,6 +178,7 @@ static void *sorcery_realtime_retrieve_fields(const struct ast_sorcery *sorcery,
 	if (!id
 		|| !(object = ast_sorcery_alloc(sorcery, type, id->value))
 		|| ast_sorcery_objectset_apply(sorcery, object, objectset)) {
+		ao2_cleanup(object);
 		return NULL;
 	}
 
@@ -232,7 +236,9 @@ static void sorcery_realtime_retrieve_multiple(const struct ast_sorcery *sorcery
 
 		objectset = sorcery_realtime_filter_objectset(objectset, &id, sorcery, type);
 
-		if (id && (object = ast_sorcery_alloc(sorcery, type, id->value)) && !ast_sorcery_objectset_apply(sorcery, object, objectset)) {
+		if (id
+			&& (object = ast_sorcery_alloc(sorcery, type, id->value))
+			&& !ast_sorcery_objectset_apply(sorcery, object, objectset)) {
 			ao2_link(objects, object);
 		}
 
@@ -262,6 +268,23 @@ static void sorcery_realtime_retrieve_regex(const struct ast_sorcery *sorcery, v
 	sorcery_realtime_retrieve_multiple(sorcery, data, type, objects, fields);
 }
 
+static void sorcery_realtime_retrieve_prefix(const struct ast_sorcery *sorcery, void *data, const char *type,
+					     struct ao2_container *objects, const char *prefix, const size_t prefix_len)
+{
+	char field[strlen(UUID_FIELD) + 6], value[prefix_len + 2];
+	RAII_VAR(struct ast_variable *, fields, NULL, ast_variables_destroy);
+
+	if (prefix_len) {
+		snprintf(field, sizeof(field), "%s LIKE", UUID_FIELD);
+		snprintf(value, sizeof(value), "%.*s%%", (int) prefix_len, prefix);
+		if (!(fields = ast_variable_new(field, value, ""))) {
+			return;
+		}
+	}
+
+	sorcery_realtime_retrieve_multiple(sorcery, data, type, objects, fields);
+}
+
 static int sorcery_realtime_update(const struct ast_sorcery *sorcery, void *data, void *object)
 {
 	struct sorcery_config *config = data;
