@@ -30,6 +30,7 @@
 
 /*** MODULEINFO
 	<depend>srtp</depend>
+	<use type="external">openssl</use>
 	<support_level>core</support_level>
 ***/
 
@@ -41,7 +42,6 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 
 #if HAVE_SRTP_VERSION > 1
 # include <srtp2/srtp.h>
-# include <srtp2/crypto_types.h>
 # include "srtp/srtp_compat.h"
 # include <openssl/rand.h>
 #else
@@ -196,7 +196,7 @@ static struct ast_srtp *res_srtp_new(void)
 		ast_free(srtp);
 		return NULL;
 	}
-	
+
 	srtp->warned = 1;
 
 	return srtp;
@@ -453,7 +453,7 @@ static int ast_srtp_protect(struct ast_srtp *srtp, void **buf, int *len, int rtc
 	if ((*len + SRTP_MAX_TRAILER_LEN) > sizeof(srtp->buf)) {
 		return -1;
 	}
-	
+
 	localbuf = rtcp ? srtp->rtcpbuf : srtp->buf;
 
 	memcpy(localbuf, *buf, *len);
