@@ -959,7 +959,8 @@ AST_TEST_DEFINE(config_hook)
 {
 	enum ast_test_result_state res = AST_TEST_FAIL;
 	enum config_hook_flags hook_flags = { 0, };
-	struct ast_flags config_flags = { CONFIG_FLAG_FILEUNCHANGED };
+	struct ast_flags config_flags = { 0 };
+	struct ast_flags reload_flags = { CONFIG_FLAG_FILEUNCHANGED };
 	struct ast_config *cfg;
 
 	switch (cmd) {
@@ -1027,7 +1028,7 @@ AST_TEST_DEFINE(config_hook)
 	 * Hook should not run
 	 */
 	hook_run = 0;
-	cfg = ast_config_load(CONFIG_FILE, config_flags);
+	cfg = ast_config_load(CONFIG_FILE, reload_flags);
 	/* Only destroy this cfg conditionally. Otherwise a crash happens. */
 	if (cfg != CONFIG_STATUS_FILEUNCHANGED) {
 		ast_config_destroy(cfg);
@@ -1457,13 +1458,19 @@ static struct aco_type global = {
 static struct aco_type global_defaults = {
 	.type = ACO_GLOBAL,
 	.item_offset = offsetof(struct test_config, global_defaults),
-	.category_match = ACO_WHITELIST,
-	.category = "^global_defaults$",
+	.category_match = ACO_WHITELIST_EXACT,
+	.category = "global_defaults",
 };
+static const char *item_blacklist[] = {
+	"global",
+	"global_defaults",
+	NULL,
+};
+
 static struct aco_type item = {
 	.type = ACO_ITEM,
-	.category_match = ACO_BLACKLIST,
-	.category = "^(global|global_defaults)$",
+	.category_match = ACO_BLACKLIST_ARRAY,
+	.category = (const char *)item_blacklist,
 	.item_alloc = test_item_alloc,
 	.item_find = test_item_find,
 	.item_offset = offsetof(struct test_config, items),
@@ -1924,4 +1931,3 @@ static int load_module(void)
 }
 
 AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Config test module");
-
